@@ -20,10 +20,10 @@
  */
 
 #include "defs.h"
+#if PDBFMTS_ENABLED
 #include "coldsync/palm.h"
 #include "coldsync/pdb.h"
-
-static double conv = 180.0 / M_PI;
+#include "grtcirc.h"
 
 #define MYNAME		"CoPilot Waypoint"
 #define MYTYPE 		0x77617970  	/* wayp */
@@ -41,8 +41,8 @@ struct record {
 static FILE *file_in;
 static FILE *file_out;
 static const char *out_fname;
-struct pdb *opdb;
-struct pdb_record *opdb_rec;
+static struct pdb *opdb;
+static struct pdb_record *opdb_rec;
 
 static void
 rd_init(const char *fname)
@@ -92,9 +92,9 @@ data_read(void)
 
 		rec = (struct record *) pdb_rec->data;
 		wpt_tmp->longitude =
-		  -pdb_read_double(&rec->longitude) * conv;
+		  DEG(-pdb_read_double(&rec->longitude));
 		wpt_tmp->latitude =
-		  pdb_read_double(&rec->latitude) * conv;
+		  DEG(pdb_read_double(&rec->latitude));
 		wpt_tmp->altitude =
 		  pdb_read_double(&rec->elevation) * .3048;
 
@@ -124,9 +124,8 @@ copilot_writewpt(const waypoint *wpt)
 
 	rec = xcalloc(sizeof(*rec)+1141,1);
 
-	pdb_write_double(&rec->latitude, wpt->latitude / conv);
-	pdb_write_double(&rec->longitude,
-		-wpt->longitude / conv);
+	pdb_write_double(&rec->latitude, RAD(wpt->latitude));
+	pdb_write_double(&rec->longitude, RAD(-wpt->longitude));
 	pdb_write_double(&rec->elevation,
 		wpt->altitude / .3048);
 	pdb_write_double(&rec->magvar, 0);
@@ -200,5 +199,8 @@ ff_vecs_t copilot_vecs = {
 	wr_deinit,
 	data_read,
 	data_write,
-	NULL
+	NULL,
+	NULL,
+	CET_CHARSET_ASCII, 0	/* CET-REVIEW */
 };
+#endif
