@@ -184,7 +184,9 @@ typedef enum {
 	gt_earth,
 	gt_locationless,
 	gt_benchmark, /* Extension to Groundspeak for GSAK */
-	gt_cito
+	gt_cito,
+	gt_ape,
+	gt_mega
 } geocache_type;
 
 typedef enum {
@@ -378,8 +380,13 @@ typedef struct {
 	double	min_alt;
 	double	max_spd;	/* Meters/sec */
 	double	min_spd;	/* Meters/sec */
+	double	avg_hrt;	/* Avg Heartrate */
+	double	avg_cad;	/* Avg Cadence */
 	time_t	start;		/* Min time */
 	time_t	end;		/* Max time */
+	int	min_hrt;        /* Min Heartrate */
+	int	max_hrt;        /* Max Heartrate */
+	int	max_cad;        /* Max Cadence */
 } computed_trkdata;
 
 /*
@@ -392,13 +399,17 @@ typedef struct {
 	double min_lon;
 } bounds;
 
+typedef struct {
+	int request_terminate;
+} posn_status;
+
 typedef void (*ff_init) (char const *);
 typedef void (*ff_deinit) (void);
 typedef void (*ff_read) (void);
 typedef void (*ff_write) (void);
 typedef void (*ff_exit) (void);
 typedef void (*ff_writeposn) (waypoint *);
-typedef waypoint * (*ff_readposn) (void);
+typedef waypoint * (*ff_readposn) (posn_status *);
 
 #ifndef DEBUG_MEM
 char * get_option(const char *iarglist, const char *argname);
@@ -713,6 +724,7 @@ char * strip_html(const utf_string*);
 char * strip_nastyhtml(const char * in);
 char * convert_human_date_format(const char *human_datef);	/* "MM,YYYY,DD" -> "%m,%Y,%d" */
 char * convert_human_time_format(const char *human_timef);	/* "HH+mm+ss"   -> "%H+%M+%S" */
+char * pretty_deg_format(double lat, double lon, char fmt, int html);   /* decimal ->  dd.dddd or dd mm.mmm or dd mm ss */
 
 /* 
  * Character encoding transformations.
@@ -777,6 +789,11 @@ void le_write32(void *pp, const unsigned i);
 double pdb_read_double(void *p);
 void pdb_write_double(void *pp, double d);
 
+double le_read_double(void *p);
+double be_read_double(void *p);
+void le_write_double(void *p, double d);
+void be_write_double(void *p, double d);
+
 /*
  * Prototypes for generic conversion routines (util.c).
  */
