@@ -439,7 +439,6 @@ tag_gs_cache(const char **attrv)
 {
 	const char **avp;
 
-	cache_descr_is_html = 0;
 	for (avp = &attrv[0]; *avp; avp+=2) {
 		if (strcmp(avp[0], "id") == 0) {
 				wpt_tmp->gc_data.id = atoi(avp[1]);
@@ -650,6 +649,8 @@ gs_type_mapping{
 	{ gt_cito, "Cache In Trash Out Event" },
 	{ gt_letterbox, "Letterbox Hybrid" },
 	{ gt_locationless, "Locationless (Reverse) Cache" },
+	{ gt_ape, "Project APE Cache" },
+	{ gt_mega, "Mega-Event Cache" },
 
 	{ gt_benchmark, "Benchmark" }, /* Not Groundspeak; for GSAK  */
 };
