@@ -33,12 +33,14 @@ static char *opt_extrude = NULL;
 static char *opt_trackdata = NULL;
 static char *opt_units = NULL;
 static char *opt_labels = NULL;
+static char *opt_max_position_points = NULL;
 
 static int export_lines;
 static int export_points;
 static int floating;
 static int extrude;
 static int trackdata;
+static int max_position_points;
 
 static int indent_level;
 
@@ -93,9 +95,22 @@ arglist_t kml_args[] = {
 	{"labels", &opt_labels,
 	 "Display labels on track and routepoints  (default = 1)",
 	 "1", ARGTYPE_BOOL, ARG_NOMINMAX },
+	{"max_position_points", &opt_max_position_points,
+	 "Retain at most this number of position points  (0 = unlimited)",
+	 "0", ARGTYPE_INT, ARG_NOMINMAX },
 	ARG_TERMINATOR
 };
 
+static 
+struct {
+	int freshness;
+	char *icon;
+} kml_tracking_icons[] = {
+ { 60, "http://maps.google.com/mapfiles/kml/pal4/icon15.png" }, // Red
+ { 30, "http://maps.google.com/mapfiles/kml/pal4/icon31.png" }, // Yellow
+ { 0, "http://maps.google.com/mapfiles/kml/pal4/icon62.png" }, // Green
+};
+
 #define MYNAME "kml"
 
 #if ! HAVE_LIBEXPAT
@@ -255,20 +270,38 @@ kml_wr_position_init(const char *fname)
 	posnfilename = fname;;
 	posnfilenametmp = xstrappend(xstrdup(fname), "-");
 	realtime_positioning = 1;
+
 	/*
 	 * 30% of our output file is whitespace.  Since parse time
 	 * matters in this mode, turn the pretty formatting off.
 	 */
 	do_indentation = 0;
-	kml_wr_init(fname);
+	kml_wr_init(posnfilenametmp);
+
+	max_position_points = atoi(opt_max_position_points);
 }
 
 static void
 kml_wr_deinit(void)
 {
 	fclose(ofd);
-	rename(posnfilenametmp, posnfilename);
-	xfree(posnfilenametmp);
+
+	if (posnfilenametmp) {
+#if __WIN32__
+		/*
+		 * This is gross.
+		 * Windows does not offer an atomic rename; we must
+		 * explictly remove the destination here which exposes
+		 * a window where a polled reader of this file could find
+		 * the file to be missing.  Windows readers will simply
+		 * have to retry on this case.
+		 */
+		unlink(posnfilename);
+#endif
+		rename(posnfilenametmp, posnfilename);
+		xfree(posnfilenametmp);
+		posnfilenametmp = NULL;
+	}
 	ofd = NULL;
 }
 
@@ -313,14 +346,13 @@ kml_write_xmle(const char *tag, const char *v)
 		for (i = 0; i < indent_level; i++) {
 			fputs("  ", ofd);
 		}
-		fprintf(ofd, "<%s>%s</%s>\n",tag, v, tag);
+		fprintf(ofd, "<%s>%s</%s>\n",tag, tmp_ent, tag);
 		xfree(tmp_ent);
 	}
 }
 
 #define hovertag(h) h ? 'h' : 'n'
-static void kml_write_bitmap_style_(const char *style, int bitmap, 
-	                               int x, int y, int width, int height, 
+static void kml_write_bitmap_style_(const char *style, const char * bitmap, 
 				       int highlighted)
 {
 	kml_write_xml(0,"<!-- %s %s style -->\n", 
@@ -331,10 +363,7 @@ static void kml_write_bitmap_style_(const char *style, int bitmap,
 		kml_write_xml(0, "<scale>1.2</scale>\n");
 	}
 	kml_write_xml(1, "<Icon>\n");
-	kml_write_xml(0, "<href>root://icons/bitmap-%d.png</href>\n", bitmap);
-	kml_write_xml(0, "<x>%d</x>\n", x);
-	kml_write_xml(0, "<w>%d</w>\n", width);
-	kml_write_xml(0, "<h>%d</h>\n", height);
+	kml_write_xml(0, "<href>%s</href>\n", bitmap);
 	kml_write_xml(-1, "</Icon>\n");
 	kml_write_xml(-1, "</IconStyle>\n");
 	kml_write_xml(-1, "</Style>\n");
@@ -344,11 +373,10 @@ static void kml_write_bitmap_style_(const char *style, int bitmap,
  * and non-highlighted version of the style to allow the icons
  * to magnify slightly on a rollover.
  */
-static void kml_write_bitmap_style(const char *style, int bitmap, 
-	                               int x, int y, int width, int height)
+static void kml_write_bitmap_style(const char *style, const char *bitmap)
 {
-	kml_write_bitmap_style_(style, bitmap, x, y, width, height, 0);
-	kml_write_bitmap_style_(style, bitmap, x, y, width, height, 1);
+	kml_write_bitmap_style_(style, bitmap, 0);
+	kml_write_bitmap_style_(style, bitmap, 1);
 
 	kml_write_xml(1, "<StyleMap id=\"%s\">\n", style);
 	kml_write_xml(1, "<Pair>\n");
@@ -378,7 +406,7 @@ static void kml_output_timestamp(const waypoint *waypointp)
  * Output the track summary.
  */
 static 
-void kml_output_trkdescription(computed_trkdata *td)
+void kml_output_trkdescription(const route_head *header, computed_trkdata *td)
 {
 	char *max_alt_units;
 	double max_alt;
@@ -395,22 +423,14 @@ void kml_output_trkdescription(computed_trkdata *td)
 	min_alt = fmt_distance(td->min_alt, &min_alt_units);
 	distance = fmt_distance(td->distance_meters, &distance_units);
 
-	/* We won't always have times. Garmin saved tracks, for example... */
-	if (td->start && td->end) {
-		char time_string[64];
-		kml_write_xml(1, "<TimeSpan>\n");
-		xml_fill_in_time(time_string, td->start, XML_LONG_TIME);
-		kml_write_xml(0, "<begin>%s</begin>\n", time_string);
-		xml_fill_in_time(time_string, td->end, XML_LONG_TIME);
-		kml_write_xml(0, "<end>%s</end>\n", time_string);
-		kml_write_xml(-1, "</TimeSpan>\n");
-	} else {
-		kml_write_xml(0, "<Snippet/>\n");
-	}
+	kml_write_xml(0, "<Snippet/>\n");
 
 	kml_write_xml(1, "<description>\n");
-	kml_write_xml(1, "<table>\n");
+	kml_write_xml(1, "<![CDATA[<table>\n");
 
+	if (header->rte_desc) {
+		TD("<b>Description</b> %s", header->rte_desc);
+	}
 	TD2("<b>Distance</b> %.1f %s", distance, distance_units);
 	if (min_alt != unknown_alt) {
 		TD2("<b>Min Alt</b> %.1f %s", min_alt, min_alt_units);
@@ -428,24 +448,52 @@ void kml_output_trkdescription(computed_trkdata *td)
 		double spd = fmt_speed(td->max_spd, &spd_units);
 		TD2("<b>Max Speed</b> %.1f %s", spd, spd_units);
 	}
-	kml_write_xml(-1, "</table>\n");
+	if (td->avg_hrt) {
+		TD("<b>Avg Heart Rate</b> %.1f bpm", td->avg_hrt);
+	}
+	if (td->min_hrt < td->max_hrt) {
+		TD("<b>Min Heart Rate</b> %d bpm", td->min_hrt);
+	}
+	if (td->max_hrt) {
+		TD("<b>Max Heart Rate</b> %d bpm", td->max_hrt);
+	}
+	if (td->avg_cad) {
+		TD("<b>Avg Cadence</b> %.1f rpm", td->avg_cad);
+	}
+	if (td->max_cad) {
+		TD("<b>Max Cadence</b> %d rpm", td->max_cad);
+	}
+
+	kml_write_xml(-1, "</table>]]>\n");
 	kml_write_xml(-1, "</description>\n");
+
+	/* We won't always have times. Garmin saved tracks, for example... */
+	if (td->start && td->end) {
+		char time_string[64];
+		kml_write_xml(1, "<TimeSpan>\n");
+		xml_fill_in_time(time_string, td->start, XML_LONG_TIME);
+		kml_write_xml(0, "<begin>%s</begin>\n", time_string);
+		xml_fill_in_time(time_string, td->end, XML_LONG_TIME);
+		kml_write_xml(0, "<end>%s</end>\n", time_string);
+		kml_write_xml(-1, "</TimeSpan>\n");
+	}
 }
 
 
 static 
 void kml_output_header(const route_head *header, computed_trkdata*td)
 {
-        kml_write_xml(1,  "<Folder>\n");
+	if (!realtime_positioning)  {
+		kml_write_xml(1,  "<Folder>\n");
+	}
 	kml_write_xmle("name", header->rte_name);
-	kml_write_xmle("desc", header->rte_desc);
+	kml_output_trkdescription(header, td);
 
-        if (export_points && header->rte_waypt_ct > 0) {
-	  kml_output_trkdescription(td);
-          // Put the points in a subfolder
-          kml_write_xml(1,  "<Folder>\n");
-          kml_write_xml(0,  "<name>Points</name>\n");
-        }
+	if (export_points && header->rte_waypt_ct > 0) {
+	       // Put the points in a subfolder
+	       kml_write_xml(1,  "<Folder>\n");
+	       kml_write_xml(0,  "<name>Points</name>\n");
+	}
 
         // Create an array for holding waypoint coordinates so that we
         // can produce a LineString at the end.
@@ -512,9 +560,10 @@ static void kml_output_point(const waypoint *waypointp, const char *style)
 		kml_write_xmle("name", waypointp->shortname);
 	}
 	kml_write_xml(0, "<Snippet/>\n");
-	kml_write_xml(0, "<styleUrl>%s</styleUrl>\n", style);
 	kml_output_description(waypointp);
 	kml_output_lookat(waypointp);
+	kml_output_timestamp(waypointp);
+	kml_write_xml(0, "<styleUrl>%s</styleUrl>\n", style);
 	kml_write_xml(1, "<Point>\n");
         if (floating) {
           kml_write_xml(0, "<altitudeMode>absolute</altitudeMode>\n");
@@ -526,8 +575,6 @@ static void kml_output_point(const waypoint *waypointp, const char *style)
 		pt->longitude, pt->latitude, pt->altitude);
 	kml_write_xml(-1, "</Point>\n");
 
-	// Timestamp
-	kml_output_timestamp(waypointp);
 
 	kml_write_xml(-1, "</Placemark>\n");
   }
@@ -545,9 +592,9 @@ static void kml_output_tailer(const route_head *header)
   // Add a linestring for this track?
   if (export_lines && point3d_list_len > 0) {
     kml_write_xml(1, "<Placemark>\n");
-    kml_write_xml(0, "<styleUrl>#lineStyle</styleUrl>\n");
     kml_write_xml(0, "<name>Path</name>\n");
 //    fprintf(ofd, "\t  <MultiGeometry>\n");
+    kml_write_xml(0, "<styleUrl>#lineStyle</styleUrl>\n");
     kml_write_xml(1, "<LineString>\n");
     if (floating) {
       kml_write_xml(0, "<altitudeMode>absolute</altitudeMode>\n");
@@ -572,7 +619,39 @@ static void kml_output_tailer(const route_head *header)
   xfree(point3d_list);
   point3d_list = NULL;
   
-  kml_write_xml(-1, "</Folder>\n");
+  if (!realtime_positioning)  {
+    kml_write_xml(-1, "</Folder>\n");
+  }
+}
+
+static 
+char *
+kml_lookup_gc_icon(const waypoint *waypointp)
+{
+	const char *icon;
+	char *rb;
+
+	/* This could be done so much better in C99 with designated
+	 * initializers...
+	 */
+	switch (waypointp->gc_data.type) {
+		case gt_traditional: icon = "2.png"; break;
+		case gt_multi: icon = "3.png"; break;
+		case gt_virtual: icon = "4.png"; break;
+		case gt_letterbox: icon = "5.png"; break;
+		case gt_event: icon = "6.png"; break;
+		case gt_ape: icon = "7.png"; break;
+		case gt_locationless: icon = "8.png"; break; // No unique icon.
+		case gt_suprise: icon = "8.png"; break;
+		case gt_webcam: icon = "11.png"; break;
+		case gt_cito: icon = "13.png"; break;
+		case gt_earth:  icon = "earthcache.png"; break;
+		case gt_mega: icon = "6.png"; break; // No unique icon yet.
+		default: icon = "8.png"; break;
+	}
+
+	xasprintf(&rb, "http://www.geocaching.com/images/kml/%s", icon);
+	return rb;
 }
 
 /*
@@ -586,7 +665,6 @@ static void kml_waypt_pr(const waypoint *waypointp)
 	kml_write_xml(1, "<Placemark>\n");
 
 	kml_write_xmle("name", waypointp->shortname);
-	kml_write_xml(0, "<styleUrl>#waypoint</styleUrl>\n");
 
 	// Description
 	if (waypointp->url && waypointp->url[0]) {
@@ -595,15 +673,63 @@ static void kml_waypt_pr(const waypoint *waypointp)
 		kml_write_xml(0,"<description>\n");
 		if (waypointp->url_link_text && waypointp->url_link_text[0])  {
 			char *olink = xml_entitize(waypointp->url_link_text);
-			kml_write_xml(0,"<a href=\"%s\">%s</a>", odesc, olink);
+			kml_write_xml(0,"<![CDATA[<a href=\"%s\">%s</a>]]>", odesc, olink);
 			xfree(olink);
 		}
 		else
 			fputs(odesc, ofd);
+
+		if (!global_opts.no_smart_icons && 
+		     waypointp->gc_data.diff && waypointp->gc_data.terr) {
+			if (waypointp->gc_data.placer) {
+				char *p = xml_entitize(waypointp->gc_data.placer);
+				fprintf(ofd, "<![CDATA[<i> by %s</i>]]>", p);
+				xfree(p);
+			}
+			fprintf(ofd, " %s (%3.1f/%3.1f)", 
+				gs_get_container(waypointp->gc_data.container),
+				waypointp->gc_data.diff / 10.0,  
+				waypointp->gc_data.terr / 10.0);
+			if (waypointp->gc_data.desc_short.utfstring) {
+				// Dont entitize it - either XML or HTML.
+				// Wrap it in a cdata and let Earth work it out.
+
+				fprintf(ofd, "<![CDATA[<p>%s</p>]]>\n", waypointp->gc_data.desc_short.utfstring);
+
+			}
+		}
 		kml_write_xml(0, "</description>\n");
 		xfree(odesc);
 	}
 
+	// Timestamp
+	kml_output_timestamp(waypointp);
+
+	// Icon - but only if it looks like a URL.
+	icon = opt_deficon ? opt_deficon : waypointp->icon_descr;
+	if (icon && strstr(icon, "://")) {
+		kml_write_xml(1, "<Style>\n");
+		kml_write_xml(1, "<IconStyle>\n");
+		kml_write_xml(1, "<Icon>\n");
+		kml_write_xml(0, "<href>%s</href>\n", icon);
+		kml_write_xml(-1, "</Icon>\n");
+		kml_write_xml(-1, "</IconStyle>\n");
+		kml_write_xml(-1, "</Style>\n");
+
+	} else if (!global_opts.no_smart_icons && waypointp->gc_data.diff && waypointp->gc_data.terr) {
+		char *is = kml_lookup_gc_icon(waypointp);
+		kml_write_xml(1, "<Style>\n");
+		kml_write_xml(1, "<IconStyle>\n");
+		kml_write_xml(1, "<Icon>\n");
+		kml_write_xml(0, "<href>%s</href>\n", is);
+		kml_write_xml(-1, "</Icon>\n");
+		kml_write_xml(-1, "</IconStyle>\n");
+		kml_write_xml(-1, "</Style>\n");
+		xfree(is);
+	} else {
+		kml_write_xml(0, "<styleUrl>#waypoint</styleUrl>\n");
+	}
+
 	// Location
 	kml_write_xml(1, "<Point>\n");
 
@@ -620,16 +746,7 @@ static void kml_waypt_pr(const waypoint *waypointp)
 		waypointp->altitude == unknown_alt ? 0.0 : waypointp->altitude);
 	kml_write_xml(-1, "</Point>\n");
 
-	// Icon - but only if it looks like a URL.
-	icon = opt_deficon ? opt_deficon : waypointp->icon_descr;
-	if (icon && strstr(icon, "://")) {
-		fprintf(ofd, "\t  <Style>\n");
-		fprintf(ofd, "\t\t<icon>%s</icon>\n", icon);
-		fprintf(ofd, "\t  </Style>\n");
-	}
 
-	// Timestamp
-	kml_output_timestamp(waypointp);
 
 	kml_write_xml(-1, "</Placemark>\n");
 }
@@ -689,70 +806,134 @@ void kml_write(void)
 
 	kml_write_xml(0,"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
 	kml_write_xml(1,"<kml xmlns=\"http://earth.google.com/kml/2.1\">\n");
-	kml_write_xml(1,"<Document xmlns:xlink=\"http://www.w3/org/1999/xlink\">\n");
+	kml_write_xml(1,"<Document xmlns:xlink=\"http://www.w3.org/1999/xlink\">\n");
 
 	now = current_time();
 	strftime(import_time, sizeof(import_time), "%c", localtime(&now));
-	kml_write_xml(0, "<name>GPS device</name>\n");
-	kml_write_xml(0, "<Snippet>Created %s</Snippet>\n", import_time);
+	if (realtime_positioning) 
+		kml_write_xml(0, "<name>GPS position</name>\n");
+	else
+		kml_write_xml(0, "<name>GPS device</name>\n");
+
+	if (now) {
+		kml_write_xml(0, "<Snippet>Created %s</Snippet>\n", import_time);
+	}
 
 	// Style settings for bitmaps
-	kml_write_bitmap_style("route", 4, 160, 0, 32, 32);
-	kml_write_bitmap_style("track", 4, 128, 0, 32, 32);
-	kml_write_bitmap_style("waypoint", 4, 160, 0, 32, 32);
+	kml_write_bitmap_style("route", "http://maps.google.com/mapfiles/kml/pal4/icon61.png");
+	kml_write_bitmap_style("track", "http://maps.google.com/mapfiles/kml/pal4/icon60.png");
+	kml_write_bitmap_style("waypoint", "http://maps.google.com/mapfiles/kml/pal4/icon61.png");
         
         // Style settings for line strings
         kml_write_xml(1, "<Style id=\"lineStyle\">\n");
         kml_write_xml(1, "<LineStyle>\n");
         kml_write_xml(0, "<color>%s</color>\n", opt_line_color);
-	kml_write_xml(0, "<width>%s</width>\n", opt_line_width);
+        kml_write_xml(0, "<width>%s</width>\n", opt_line_width);
         kml_write_xml(-1, "</LineStyle>\n");
         kml_write_xml(-1, "</Style>\n");
 
-	kml_write_xml(1, "<Folder>\n");
-	kml_write_xml(0, "<name>Waypoints</name>\n");
+	if (!realtime_positioning) {
+		kml_write_xml(1, "<Folder>\n");
+		kml_write_xml(0, "<name>Waypoints</name>\n");
+	}
 
 	waypt_disp_all(kml_waypt_pr);
 
-	kml_write_xml(-1, "</Folder>\n");
+	if (!realtime_positioning) {
+		kml_write_xml(-1, "</Folder>\n");
+	}
 
 	// Output trackpoints
-	kml_write_xml(1,  "<Folder>\n");
-	kml_write_xml(0,  "<name>Tracks</name>\n");
+	if (!realtime_positioning) {
+		kml_write_xml(1,  "<Folder>\n");
+		kml_write_xml(0,  "<name>Tracks</name>\n");
+	}
 	track_disp_all(kml_track_hdr, kml_track_tlr, kml_track_disp);
-	kml_write_xml(-1,  "</Folder>\n");
-  
+	if (!realtime_positioning) {
+		kml_write_xml(-1,  "</Folder>\n");
+	}
+
 	// Output routes
-	kml_write_xml(1,  "<Folder>\n");
-	kml_write_xml(0,  "<name>Routes</name>\n");
-	route_disp_all(kml_route_hdr, kml_route_tlr, kml_route_disp);
-	kml_write_xml(-1,  "</Folder>\n");
+	if (!realtime_positioning) {
+		kml_write_xml(1,  "<Folder>\n");
+		kml_write_xml(0,  "<name>Routes</name>\n");
+		route_disp_all(kml_route_hdr, kml_route_tlr, kml_route_disp);
+		kml_write_xml(-1,  "</Folder>\n");
+	}
 
 	kml_write_xml(-1, "</Document>\n");
 	kml_write_xml(-1, "</kml>\n");
 }
 
+/*
+ * This depends on the table being sorted correctly.
+ */
+static 
+char *
+kml_get_posn_icon(int freshness)
+{
+	int i;
+	for (i = 0; i < sizeof(kml_tracking_icons) / sizeof(kml_tracking_icons[0]); i++) {
+		if (freshness >= kml_tracking_icons[i].freshness)
+			return kml_tracking_icons[i].icon;
+	}
+	return "http://maps.google.com/mapfiles/kml/pal3/icon59.png";
+}
+
+
+static route_head *posn_trk_head = NULL;
+
 static void
 kml_wr_position(waypoint *wpt)
 {
-	static route_head *trk_head = NULL;
-	waypoint *t = waypt_dupe(wpt);
+	static time_t last_valid_fix;
+
+	if (!posn_trk_head) {
+		posn_trk_head = route_head_alloc();
+		track_add_head(posn_trk_head);
+	}
+
+	if (last_valid_fix == 0) last_valid_fix = current_time();
 
 	/* We want our waypoint to have a name, but not our trackpoint */	
 	if (!wpt->shortname) {
-		wpt->shortname = xstrdup("Position");
+		if (wpt->fix == fix_none) {
+			wpt->shortname = xstrdup("ESTIMATED Position");
+		} else {
+			wpt->shortname = xstrdup("Position");
+		}
 	}
 
-	if (!trk_head) {
-		trk_head = route_head_alloc();
-		track_add_head(trk_head);
+	switch (wpt->fix) {
+		case fix_none:
+			if (wpt->shortname) {
+				xfree (wpt->shortname);
+			}
+			wpt->shortname = xstrdup("ESTIMATED Position");
+			break;
+		case fix_unknown:
+			break;
+		default:
+			last_valid_fix = wpt->creation_time;
 	}
 
-	track_add_wpt(trk_head, t);
+	wpt->icon_descr = kml_get_posn_icon(wpt->creation_time - last_valid_fix);
+
+	track_add_wpt(posn_trk_head, waypt_dupe(wpt));
 
 	waypt_add(wpt);
 	kml_write();
 	waypt_del(wpt);
+
+	/*
+	 * If we are keeping only a recent subset of the trail, trim the
+	 * head here.
+ 	 */
+	while (max_position_points && 
+	       (posn_trk_head->rte_waypt_ct >= max_position_points)) {
+		waypoint *tonuke = (waypoint *) QUEUE_FIRST(&posn_trk_head->waypoint_list);
+		track_del_wpt(posn_trk_head, tonuke);
+	}
 }
 
 ff_vecs_t kml_vecs = {
