@@ -2,7 +2,7 @@
 	Access to Lowrance USR files.
 	Contributed to gpsbabel by Jason Rust (jrust at rustyparts.com)
 
-	Copyright (C) 2005 Robert Lipe, robertlipe@usa.net
+	Copyright (C) 2005, 2006 Robert Lipe, robertlipe@usa.net
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -131,12 +131,36 @@ const lowranceusr_icon_mapping_t lowranceusr_icon_value_table[] = {
 	{ 10019, "Shower" },		/* person */
 	{ DEF_ICON, "Tunnel" },
 
+	/* This list comes from 'wifinder' from ifinder H20 Color */
+	
+        { 10062, "Interesting Land Feature" },
+        { 10063, "Global Location" },
+        { 10064, "Note" },
+        { 10065, "Ghost" },
+        { 10066, "Letter" },
+        { 10067, "Multi-Treasure" },
+        { 10068, "Mystery Or Puzzle" },
+        { 10069, "Treasure" },
+        { 10070, "Webmail" },
+        { 10071, "Sun" },
+        { 10072, "Musical Note" },
+        { 10073, "Camera/Movie Theater" },
+        { 10074, "Star" },
+        { 10075, "Coffee Mug" },
+        { 10076, "Books" },
+        { 10077, "Historical Marker" },
+        { 10078, "Tools/Repair" },
+        { 10079, "Favorite" },
+        { 10080, "Arena" },
+        { 10081, "Golf Course" },
+        { 10082, "Money/Atm" },
+
 	{	 -1, NULL }
 };
 
 static FILE *file_in;
 static FILE *file_out;
-static void *mkshort_handle;
+static short_handle mkshort_handle;
 
 static unsigned short waypt_out_count;
 static unsigned int trail_count, lowrance_route_count;
@@ -156,9 +180,10 @@ static char *seg_break;
 #define DEGREESTORADIANS	0.017453292
 #define SECSTO2000			946713600
 #define MAX_TRAIL_POINTS 9999
+#define UNKNOWN_USR_ALTITUDE	METERS_TO_FEET(-10000) /* -10000ft is how the unit stores unknown */
 
 /* Jan 1, 2000 00:00:00 */
-struct tm base_time = { 0, 0, 0, 1, 0, 100, 5, 1, -1 };
+const time_t base_time_secs = 946706400;
 
 static
 size_t
@@ -194,11 +219,22 @@ long int
 lowranceusr_find_icon_number_from_desc(const char *desc)
 {
 	const lowranceusr_icon_mapping_t *i;
+	int n;
 
 	if (!desc) {
 		return DEF_ICON;
 	}
 
+	/*
+	 * If we were given a numeric icon number as a description 
+	 * (i.e. 8255), just return that.
+	 */
+	n = atoi(desc);
+	if (n)  {
+		return n;
+	}
+
+
 	for (i = lowranceusr_icon_value_table; i->icon; i++) {
 		if (case_ignore_strcmp(desc,i->icon) == 0) {
 			return i->value;
@@ -215,7 +251,8 @@ lowranceusr_fread(void *buff, size_t size, size_t members, FILE * fp)
 	br = fread(buff, size, members, fp);
 
 	if (br != members) {
-		fatal(MYNAME ": requested to read %d bytes, read %d bytes.\n", members, br);
+		fatal(MYNAME ": requested to read %lu bytes, read %lu bytes.\n", 
+		            (unsigned long) members, (unsigned long) br);
 	}
 
 	return (br);
@@ -224,12 +261,12 @@ lowranceusr_fread(void *buff, size_t size, size_t members, FILE * fp)
 static
 arglist_t lowranceusr_args[] = {
 	{"ignoreicons", &ignoreicons, "Ignore event marker icons",
-	 NULL, ARGTYPE_BOOL },
+	 NULL, ARGTYPE_BOOL, ARG_NOMINMAX },
 	{"merge", &merge, "(USR output) Merge into one segmented track",
-	NULL, ARGTYPE_BOOL },
+	NULL, ARGTYPE_BOOL, ARG_NOMINMAX },
 	{"break", &seg_break, "(USR input) Break segments into separate tracks",
-	NULL, ARGTYPE_BOOL },
-	{NULL, NULL, NULL, NULL, 0}
+	NULL, ARGTYPE_BOOL, ARG_NOMINMAX },
+	ARG_TERMINATOR
 };
 
 static void
@@ -256,7 +293,7 @@ static void
 wr_deinit(void)
 {
 	fclose(file_out);
-	mkshort_del_handle(mkshort_handle);
+	mkshort_del_handle(&mkshort_handle);
 }
 
 /**
@@ -290,7 +327,6 @@ lowranceusr_parse_waypt(waypoint *wpt_tmp)
 	long int TextLen;
 	time_t waypt_time;
 	short waypt_type;
-	time_t base_time_secs = mktime(&base_time);
 
 	lowranceusr_fread(&buff[0], 4, 1, file_in);
 	wpt_tmp->latitude = lat_mm_to_deg(le_read32(&buff[0]));
@@ -298,9 +334,13 @@ lowranceusr_parse_waypt(waypoint *wpt_tmp)
 	wpt_tmp->longitude = lon_mm_to_deg(le_read32(&buff[0]));
 	lowranceusr_fread(&buff[0], 4, 1, file_in);
 	wpt_tmp->altitude = FEET_TO_METERS(le_read32(&buff[0]));
+	if (wpt_tmp->altitude <= UNKNOWN_USR_ALTITUDE) {
+		wpt_tmp->altitude = unknown_alt;
+	}
 	lowranceusr_fread(&buff[0], 4, 1, file_in);
 	TextLen = buff[0];
-	lowranceusr_fread(&buff[0], TextLen, 1, file_in);
+	if (TextLen)
+		lowranceusr_fread(&buff[0], TextLen, 1, file_in);
 	buff[TextLen] = '\0';
 	wpt_tmp->shortname = xstrdup(buff);
 
@@ -332,6 +372,12 @@ lowranceusr_parse_waypt(waypoint *wpt_tmp)
 	/* Symbol ID */
 	lowranceusr_fread(&buff[0], 4, 1, file_in);
 	wpt_tmp->icon_descr = lowranceusr_find_desc_from_icon_number(le_read32(&buff[0]));
+	if (!wpt_tmp->icon_descr[0]) {
+		char nbuf[10];
+		snprintf(nbuf, sizeof(nbuf), "%d", le_read32(buff));
+		wpt_tmp->wpt_flags.icon_descr_is_dynamic = 1;
+		wpt_tmp->icon_descr = xstrdup(nbuf);
+	}
 
 	/* Waypoint Type (USER, TEMPORARY, POINT_OF_INTEREST) */
 	lowranceusr_fread(&buff[0], 2, 1, file_in);
@@ -525,7 +571,7 @@ if (global_opts.debug_level >= 1)
 					track_add_head(trk_tmp);
 					trk_head = trk_tmp;
 				}
-				route_add_wpt(trk_head, wpt_tmp);
+				track_add_wpt(trk_head, wpt_tmp);
 			
 if (global_opts.debug_level >= 1)
 	printf("LOWRANCE parse_trails: Trail pt lat %f lon %f\n", wpt_tmp->latitude, wpt_tmp->longitude);
@@ -592,7 +638,10 @@ lowranceusr_waypt_disp(const waypoint *wpt)
 	char *name;
 	char *comment;
 	int alt = METERS_TO_FEET(wpt->altitude);
-	time_t base_time_secs = mktime(&base_time);
+
+	if (alt == unknown_alt) {
+		alt = UNKNOWN_USR_ALTITUDE;
+	}
 
 	Lat = lat_deg_to_mm(wpt->latitude);
 	my_fwrite4(&Lat, file_out);
@@ -946,5 +995,6 @@ ff_vecs_t lowranceusr_vecs = {
 	data_read,
 	data_write,
 	NULL, 
-	lowranceusr_args
+	lowranceusr_args,
+	CET_CHARSET_ASCII, 0	/* CET-REVIEW */
 };
