@@ -50,6 +50,8 @@ static int route_out_count;
 static int waypoint_read_count;
 static int wpt_len = 8;
 static const char *curfname;
+static int extension_hint;
+
 /*
  * Magellan's firmware is *horribly* slow to send the next packet after
  * we turn around an ack while we are reading from the device.  It's
@@ -197,6 +199,7 @@ pid_to_model_t pid_to_model[] =
 	{ mm_sportrak, 45, "Mystic" },
 	{ mm_meridian, 46, "MobileMapper" },
 	{ mm_meridian, 110, "Explorist 100" },
+	{ mm_meridian, 111, "Explorist 200" },
 	{ mm_unknown, 0, NULL }
 };
 
@@ -457,16 +460,24 @@ retry:
 			waypt_status_disp(waypoint_read_count, 
 					waypoint_read_count);
 		}
-		switch (objective)
-		{
-			case wptdata:
+
+		if (extension_hint) {
+			if (extension_hint == WPTDATAMASK) {
 				waypt_add(wpt);
-				break;
-			case rtedata:
-				ENQUEUE_TAIL(&rte_wpt_tmp, &wpt->Q);
-				break;
-			default:
-				break;
+			} else if (extension_hint == RTEDATAMASK) {
+					ENQUEUE_TAIL(&rte_wpt_tmp, &wpt->Q);
+			}
+		} else {
+			switch (objective) {
+				case wptdata:
+					waypt_add(wpt);
+					break;
+				case rtedata:
+					ENQUEUE_TAIL(&rte_wpt_tmp, &wpt->Q);
+					break;
+				default:
+					break;
+			}
 		}
 	}
 	if (strncmp(ibuf, "$PMGNTRK,", 7) == 0) {
@@ -709,6 +720,7 @@ mag_serial_init_common(const char *portname)
 static void
 mag_rd_init_common(const char *portname)
 {
+	char *ext;
 	waypoint_read_count = 0;
 
 	if (bs) {
@@ -735,6 +747,24 @@ mag_rd_init_common(const char *portname)
 		curfname = portname;
 	}
 
+	/*
+	 * I'd rather not derive behaviour from filenames but since
+	 * we can't otherwise tell if we should put a WPT on the route
+	 * queue or the WPT queue in the presence of (-w -r -t) we 
+	 * divine a hint from the filename extension when we can.
+	 */
+	ext = strrchr(curfname, '.');
+	if (ext) {
+		ext++;
+		if (0 == case_ignore_strcmp(ext, "upt")) {
+			extension_hint = WPTDATAMASK;
+		} else if (0 == case_ignore_strcmp(ext, "log")) {
+			extension_hint = TRKDATAMASK;
+		} else if (0 == case_ignore_strcmp(ext, "rte")) {
+			extension_hint = RTEDATAMASK;
+		} 
+	}
+
 	return;
 }
 
@@ -1236,7 +1266,7 @@ mag_waypt_pr(const waypoint *waypointp)
 	if (odesc && /* !is_file && */ (wptcmtcnt++ >= wptcmtcnt_max))
 		odesc[0] = 0;
 
-	sprintf(obuf, "PMGNWPL,%4.3f,%c,%09.3f,%c,%07.lf,M,%-.*s,%-.46s,%s",
+	sprintf(obuf, "PMGNWPL,%4.3f,%c,%09.3f,%c,%07.0f,M,%-.*s,%-.46s,%s",
 		lat, ilat < 0 ? 'S' : 'N',
 		lon, ilon < 0 ? 'W' : 'E',
 		waypointp->altitude == unknown_alt ?
@@ -1304,7 +1334,7 @@ void mag_track_disp(const waypoint *waypointp)
 	lon = (lon_deg * 100.0 + lon);
 	lat = (lat_deg * 100.0 + lat);
 
-	sprintf(obuf,"PMGNTRK,%4.3f,%c,%09.3f,%c,%05.f,%c,%06d.%02d,A,,%06d", 
+	sprintf(obuf,"PMGNTRK,%4.3f,%c,%09.3f,%c,%05.0f,%c,%06d.%02d,A,,%06d", 
 		lat, ilat < 0 ? 'S' : 'N',
 		lon, ilon < 0 ? 'W' : 'E',
 		waypointp->altitude == unknown_alt ?
