@@ -154,6 +154,7 @@ static struct tm tm;
 static waypoint * curr_waypt = NULL;
 static waypoint * last_waypt = NULL;
 static void * gbser_handle;
+static const char *posn_fname;
 
 static int without_date;	/* number of created trackpoints without a valid date */
 static struct tm opt_tm;	/* converted "date" parameter */
@@ -178,7 +179,7 @@ static int getposn;
 static time_t last_time = -1;
 static double last_read_time;   /* Last timestamp of GGA or PRMC */
 
-static waypoint * nmea_rd_posn(void);
+static waypoint * nmea_rd_posn(posn_status *);
 static void nmea_rd_posn_init(const char *fname);
 
 arglist_t nmea_args[] = {
@@ -225,8 +226,9 @@ nmea_rd_init(const char *fname)
  	 */
  	if (getposn) {
  		waypoint *wpt;
+		posn_status st;
  		nmea_rd_posn_init(fname);
- 		wpt = nmea_rd_posn();
+ 		wpt = nmea_rd_posn(&st);
  		if (!wpt) {
  			return;
  		}
@@ -373,7 +375,12 @@ gpgga_parse(char *ibuf)
 		&lngdeg,&lngdir,
 		&fix,&nsats,&hdop,&alt,&altunits);
 
-	if (fix == 0) {
+	/*
+	 * In serial mode, allow the fix with an invalid position through
+	 * as serial units will often spit a remembered position up and
+	 * that is more comfortable than nothing at all...
+	 */
+	if ((fix == 0) && (read_mode != rm_serial)) {
 		return;
 	}
 
@@ -402,6 +409,9 @@ gpgga_parse(char *ibuf)
 	waypt->hdop 	= hdop;
 
 	switch (fix) {
+		case 0:
+			waypt->fix = fix_none;
+			break;
 		case 1:
 			waypt->fix  = (nsats>3)?(fix_3d):(fix_2d);
 			break;
@@ -808,18 +818,19 @@ nmea_rd_posn_init(const char *fname)
 		read_mode = rm_serial;
 		gbser_set_speed(gbser_handle, 4800);
 	} else {
-		fatal("Could not open %s\n", fname);
+		fatal(MYNAME ": Could not open '%s' for position tracking.\n", fname);
 	}
 
 	if (opt_baud) {
 		if (!gbser_set_speed(gbser_handle, atoi(opt_baud))) {
-			fatal("Unable to set baud rate %s\n", opt_baud);
+			fatal(MYNAME ": Unable to set baud rate %s\n", opt_baud);
 		}
 	}
+	posn_fname = fname;
 }
 
 static waypoint *
-nmea_rd_posn(void)
+nmea_rd_posn(posn_status *posn_status)
 {
 	char ibuf[1024];
 	static double lt = -1;
@@ -839,7 +850,7 @@ nmea_rd_posn(void)
 			warning( "READ: %s\n", ibuf);
 		}
 		if (rv < 0) {
-			fatal("No data received.\n");
+			fatal(MYNAME ": No data received on %s.\n", posn_fname);
 		}
 		nmea_parse_one_line(ibuf);
 		if (lt != last_read_time) {
