@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: shpopen.c,v 1.2 2004/09/27 01:13:58 robertl Exp $
+ * $Id: shpopen.c,v 1.4 2006/07/13 03:27:54 robertl Exp $
  *
  * Project:  Shapelib
  * Purpose:  Implementation of core Shapefile read/write functions.
@@ -34,6 +34,13 @@
  ******************************************************************************
  *
  * $Log: shpopen.c,v $
+ * Revision 1.4  2006/07/13 03:27:54  robertl
+ * Andy Armstrong turns on -Wall for GCC builds and kills about a sequillion warnings.  Most of them aren't "real", but it's still a good thing to clean up.
+ * (I hope I don't regret this before 1.3.1...)
+ *
+ * Revision 1.3  2006/05/07 02:14:35  robertl
+ * Make shapefile and all palm pdb formats deselectable at build time.
+ *
  * Revision 1.2  2004/09/27 01:13:58  robertl
  * warning fixes in shapelib.  From Alexander Stohr.
  *
@@ -164,10 +171,12 @@
  *
  */
 
-static char rcsid[] = 
-  "$Id: shpopen.c,v 1.2 2004/09/27 01:13:58 robertl Exp $";
+/*static char rcsid[] = 
+  "$Id: shpopen.c,v 1.4 2006/07/13 03:27:54 robertl Exp $";*/
 
 #include "shapefil.h"
+#include "config.h"
+#if SHAPELIB_ENABLED
 
 #include <math.h>
 #include <limits.h>
@@ -1870,3 +1879,4 @@ SHPRewindObject( SHPHandle hSHP, SHPObject * psObject )
 
     return bAltered;
 }
+#endif
