@@ -145,20 +145,15 @@ static double
 tpo_fread_double(FILE *fp)
 {
 	unsigned char buf[8];
-	unsigned char sbuf[8];
-
 	tpo_fread(buf, 1, 8, fp);
-	le_read64(sbuf, buf);
-	return *(double *)sbuf;
+	return le_read_double(buf);
 }
 
 static void
 tpo_fwrite_double(double x, FILE *fp)
 {
-	unsigned char *cptr = (unsigned char *)&x;
 	unsigned char cbuf[8];
-
-	le_read64(cbuf, cptr);
+	le_write_double(cbuf,x);
 	fwrite(cbuf, 8, 1, fp);
 }
 
@@ -563,7 +558,7 @@ int tpo_find_block(unsigned int block_desired)
 	
         // Read record type
         block_type = tpo_read_32();
-//printf("Block: %x\n", block_type);
+//printf("Block: %08x\tat offset: %08x\n", block_type, block_offset);
 
         // Read offset to next record
         block_offset = tpo_read_32();
@@ -696,7 +691,7 @@ void tpo_process_tracks(void)
             sprintf(track_name, "TRK %d", ii+1);
         }
 		track_temp->rte_name = track_name;
-//printf("Track Name: %s\n", track_name);
+//printf("\nTrack Name: %s  ", track_name);
 
         // Route description
 //        track_temp->rte_desc = NULL;
@@ -735,37 +730,52 @@ void tpo_process_tracks(void)
                 lat = le_read32(buf+jj);
                 jj+=4;
 
-                // Peek to see if next is a lonscale
-                if(jj+3<track_byte_count && !buf[jj+3] && buf[jj] !=
-                        0x88 && buf[jj+1] != 0x88 && buf[jj+2] != 0x88) {
+//printf("L");
+
+                // Peek to see if next is a lonscale.  Note that it
+                // can begin with 0x88, which is confusing.  Here we
+                // allow up to 16-bits of offset, so two of the
+                // bytes must be 0x00 for us to recognize it.
+                if(jj+3<track_byte_count
+                        && !buf[jj+3]
+                        && !buf[jj+2]) {
 
                     lonscale = le_read32(buf+jj);
+//printf(" LONSCALE:");
+//printf("%02x%02x%02x%02x", buf[jj], buf[jj+1], buf[jj+2], buf[jj+3]);
                     jj+=4;
                 }
-                // Peek to see if next is a latscale
-                if(jj+3<track_byte_count && !buf[jj+3] && buf[jj] !=
-                        0x88 && buf[jj+1] != 0x88 && buf[jj+2] != 0x88) {
+                // Peek to see if next is a latscale.  Note that it
+                // can begin with 0x88, which is confusing.  Here we
+                // allow up to 16-bits of offset, so two of the
+                // bytes must be 0x00 for us to recognize it.
+                if(jj+3<track_byte_count
+                        && !buf[jj+3]
+                        && !buf[jj+2]) {
 
                     latscale = le_read32(buf+jj);
+//printf(" LATSCALE:");
+//printf("%02x%02x%02x%02x ", buf[jj], buf[jj+1], buf[jj+2], buf[jj+3]);
                     jj+=4;
                 }
                 llvalid = 1;
 
-//printf("LL");
-
                 waypoint_temp = tpo_convert_ll(lat, lon);
                 route_add_wpt(track_temp, waypoint_temp);
                 waypoint_count++;
             }
 
-            // We have a lonlat, let's see if it's time to get a new
-            // one
+            // Check whether there's a lonlat coming up instead of
+            // offsets.
             else if (buf[jj] == 0x88) {
                 jj++;
                 llvalid = 0;
             }
 
+            // Check whether there's a lonlat + lonscale/latscale
+            // combo embedded in this track next.
             else if (buf[jj] == 0x00) {
+//printf(" ZERO ");
                 jj++;
                 llvalid = 0;
             }
@@ -864,6 +874,7 @@ void tpo_process_waypoints(void)
 //UNKNOWN DATA LENGTH
         // Fetch name length
         name_length = tpo_read_int();
+//printf("\nName Length: %d\n", name_length);
         if (name_length) {
             waypoint_name = xmalloc(name_length+1);
             waypoint_name[0] = '\0';
@@ -874,7 +885,7 @@ void tpo_process_waypoints(void)
             waypoint_name = xmalloc(15);
             sprintf(waypoint_name, "WPT %d", ii+1);
         }
-//printf("Waypoint Name: %s\n", waypoint_name);
+//printf("\tWaypoint Name: %s\n", waypoint_name);
 
 //UNKNOWN DATA LENGTH
         (void)tpo_read_int();
@@ -893,11 +904,12 @@ void tpo_process_waypoints(void)
         if (altitude == 0xfffd000c) // Unknown altitude
             altitude = 0;
         waypoint_temp->altitude = altitude / 100;   // Meters
-//printf("Altitude: %d\n", waypoint_temp->altitude);
+//printf("\tAltitude: %1.0f meters\n", waypoint_temp->altitude);
 
 //UNKNOWN DATA LENGTH
         // Fetch comment length
         name_length = tpo_read_int();
+//printf("\tComment length: %d\n", name_length); 
         if (name_length) {
             char *comment;
  
@@ -906,7 +918,7 @@ void tpo_process_waypoints(void)
             tpo_fread(comment, 1, name_length, tpo_file_in);
             comment[name_length] = '\0';  // Terminator
             waypoint_temp->description = comment;
-//printf("Comment: %s\n", waypoint_name);
+//printf("\tComment: %s\n", waypoint_name);
         }
         else {
 //            waypoint_temp->description = NULL;
@@ -927,16 +939,20 @@ void tpo_process_waypoints(void)
         waypt_add(waypoint_temp);
 
 //UNKNOWN DATA LENGTH
-        (void)tpo_read_int();
+//        (void)tpo_read_int();
+	(void)tpo_read_8();
 
 //UNKNOWN DATA LENGTH
-        (void)tpo_read_int();
+//        (void)tpo_read_int();
+	(void)tpo_read_8();
 
 //UNKNOWN DATA LENGTH
-        (void)tpo_read_int();
+//        (void)tpo_read_int();
+	(void)tpo_read_8();
 
 //UNKNOWN DATA LENGTH
-        (void)tpo_read_int();
+//        (void)tpo_read_int();
+	(void)tpo_read_8();
     }
 }
 
