@@ -27,8 +27,11 @@
 #include <stdarg.h>
 #include <time.h>
 
-static int i_am_little_endian = -1;
-static int doswap(void);
+#if defined WORDS_BIGENDIAN
+# define i_am_little_endian 0
+#else
+# define i_am_little_endian 1
+#endif
 
 #ifdef DEBUG_MEM
 #define DEBUG_FILENAME "/tmp/gpsbabel.debug"
@@ -287,24 +290,89 @@ xfputs(const char *errtxt, const char *s, FILE *stream)
  * Allocate a string using a format list with optional arguments
  */
 
+int
+xvasprintf(char **strp, const char *fmt, va_list args)
+{
+	/* From http://perfec.to/vsnprintf/pasprintf.c */
+/* size of first buffer malloc; start small to exercise grow routines */
+#define	FIRSTSIZE	64
+	char *buf = NULL;
+	int bufsize;
+	char *newbuf;
+	size_t nextsize = 0;
+	int outsize;
+
+	bufsize = 0;
+	for (;;) {
+		if (bufsize == 0) {
+			if ((buf = xmalloc(FIRSTSIZE)) == NULL) {
+				*strp = NULL;
+				return -1;
+			}
+			bufsize = 1;
+		} else if ((newbuf = xrealloc(buf, nextsize)) != NULL) {
+			buf = newbuf;
+			bufsize = nextsize;
+		} else {
+			xfree(buf);
+			*strp = NULL;
+			return -1;
+		}
+
+		outsize = vsnprintf(buf, bufsize, fmt, args);
+
+		if (outsize == -1) {
+			/* Clear indication that output was truncated, but no
+			 * clear indication of how big buffer needs to be, so
+			 * simply double existing buffer size for next time.
+			 */
+			nextsize = bufsize * 2;
+
+		} else if (outsize == bufsize) {
+			/* Output was truncated (since at least the \0 could
+			 * not fit), but no indication of how big the buffer
+			 * needs to be, so just double existing buffer size
+			 * for next time.
+			 */
+			nextsize = bufsize * 2;
+
+		} else if (outsize > bufsize) {
+			/* Output was truncated, but we were told exactly how
+			 * big the buffer needs to be next time. Add two chars
+			 * to the returned size. One for the \0, and one to
+			 * prevent ambiguity in the next case below.
+			 */
+			nextsize = outsize + 2;
+
+		} else if (outsize == bufsize - 1) {
+			/* This is ambiguous. May mean that the output string
+			 * exactly fits, but on some systems the output string
+			 * may have been trucated. We can't tell.
+			 * Just double the buffer size for next time.
+			 */
+			nextsize = bufsize * 2;
+
+		} else {
+			/* Output was not truncated */
+			break;
+		}
+	}
+	*strp = buf;
+	return 0;
+}
+
 int
 xasprintf(char **strp, const char *fmt, ...)
 {
 	va_list args;
-	int res;
-	
+	int rval;
+
 	va_start(args, fmt);
-	res = vsnprintf((char *)NULL, 0, fmt, args);
-	*strp = xmalloc(res + 1);
-	va_end(args);
-	
-	va_start(args, fmt);
-	res = vsnprintf(*strp, res + 1, fmt, args);
+	rval = xvasprintf(strp, fmt, args);
 	va_end(args);
 
-	is_fatal(res < 0, "(internal): vsnprintf returned %d!", res);
+	return rval;
 	
-	return res;
 }
 
 /* 
@@ -596,8 +664,6 @@ le_read64(void *dest, const void *src)
 	char *cdest = dest;
 	const char *csrc = src;
 
-	doswap(); /* make sure i_am_little_endian is initialized */
-
 	if (i_am_little_endian) {
 		memcpy(dest, src, 8);
 	} else {
@@ -795,56 +861,65 @@ get_cache_icon(const waypoint *waypointp)
 	return NULL;
 }
 
-static int doswap()
-{
-  if (i_am_little_endian < 0)
-  {
-	/*	On Intel, Vax and MIPs little endian, -1.0 maps to the bytes
-		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0x3f and on Motorola,
-		SPARC, ARM, and PowerPC, it maps to
-		0x3f, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00.
-	*/
-	double d = 1.0;
-	char c[8];
-	memcpy(c, &d, 8);
-	i_am_little_endian = (c[0] == 0);
-  }
-  return i_am_little_endian;
-}
-
 double
-pdb_read_double(void* ptr)
+endian_read_double(void* ptr, int read_le)
 {
   double ret;
   char r[8];
+  void *p;
   int i;
-  doswap(); /* make sure i_am_little_endian is initialized */
-  for (i = 0; i < 8; i++)
-  {
-	int j = (i_am_little_endian)?(7-i):i;
-	r[i] = ((char*)ptr)[j];
+  
+  if ( i_am_little_endian == read_le ) {
+	  p = ptr;
   }
-  memcpy(&ret, r, 8);
+  else {
+	  for (i = 0; i < 8; i++)
+	  {
+		r[i] = ((char*)ptr)[7-i];
+	  }
+	  p = r;
+  }
+  
+  memcpy(&ret, p, 8);
   return ret;
 }
 
 void
-pdb_write_double(void* ptr, double d)
+endian_write_double(void* ptr, double d, int write_le)
 {
-  char r[8];
+  char *r = (char *)(void *)&d;
   int i;
   char *optr = ptr;
 
-  memcpy(r, &d, 8);
-  doswap(); /* make sure i_am_little_endian is initialized */
-  for (i = 0; i < 8; i++)
-  {
-	int j = (i_am_little_endian)?(7-i):i;
-	*optr++ = r[j];
+  if ( i_am_little_endian == write_le ) {
+	  memcpy( ptr, &d, 8);
+  }
+  else {
+	  for (i = 0; i < 8; i++)
+	  {
+		*optr++ = r[7-i];
+	  }
   }
-  return;
 }
 
+double
+pdb_read_double( void *ptr ) {return endian_read_double(ptr, 0);}
+
+double 
+le_read_double( void *ptr ) {return endian_read_double(ptr,1);}
+
+double 
+be_read_double( void *ptr ) {return endian_read_double(ptr,0);}
+
+void
+pdb_write_double( void *ptr, double d ) {endian_write_double(ptr,d,0);}
+
+void
+le_write_double( void *ptr, double d ) {endian_write_double(ptr,d,1);}
+
+void 
+be_write_double( void *ptr, double d ) {endian_write_double(ptr,d,0);}
+
 /* Magellan and PCX formats use this DDMM.mm format */
 double ddmm2degrees(double pcx_val) {
 	double minutes;
@@ -1135,10 +1210,52 @@ convert_human_time_format(const char *human_timef)
 	return result;
 }
 
+
+/* 
+ * Return a decimal degree pair as
+ * DD.DDDDD  DD MM.MMM or DD MM SS.S
+ * fmt = ['d', 'm', 's']
+ * html = 1 for html output otherwise text
+ */
+char *
+pretty_deg_format(double lat, double lon, char fmt, int html) 
+{
+	double  latmin, lonmin, latsec, lonsec;
+	int     latint, lonint;
+	char	latsig, lonsig;
+	char	*result;
+	latsig = lat < 0 ? 'S':'N';
+	lonsig = lon < 0 ? 'W':'E';
+	latint = abs((int) lat);
+  	lonint = abs((int) lon);
+	latmin = 60.0 * (fabs(lat) - latint);
+	lonmin = 60.0 * (fabs(lon) - lonint);
+	latsec = 60.0 * (latmin - floor(latmin));
+	lonsec = 60.0 * (lonmin - floor(lonmin));
+	if (fmt == 'd') { /* ddd */
+		xasprintf ( &result, "%c%6.5f%s %c%6.5f%s",
+			latsig, fabs(lat), html?"&deg;":"", 
+			lonsig, fabs(lon), html?"&deg;":"" );
+	}
+	else if (fmt == 's') { /* dms */
+		xasprintf ( &result, "%c%d%s%02d'%04.1f\" %c%d%s%02d'%04.1f\"",
+                        latsig, latint, html?"&deg;":" ", (int)latmin, latsec,
+			lonsig, lonint, html?"&deg;":" ", (int)lonmin, lonsec);
+	}
+	else { /* default dmm */
+		xasprintf ( &result,  "%c%d%s%06.3f %c%d%s%06.3f",
+			latsig, latint, html?"&deg;":" ", latmin, 
+			lonsig, lonint, html?"&deg;":" ", lonmin);
+	} 
+	return result;
+}
+
+
+
 /* 
  * Get rid of potentially nasty HTML that would influence another record
  * that includes;
- * <body> - to stop backgrounds from being loaded
+ * <body> - to stop backgrounds/background colours from being loaded
  * </body> and </html>- stop processing altogether
  * <style> </style> - stop overriding styles for everything
  */
@@ -1151,19 +1268,28 @@ strip_nastyhtml(const char * in)
 	sp = returnstr = xstrdup(in);
 	lcp = lcstr = strlower(xstrdup(in));
 	
-	while (lcp = strstr(lcstr, "<body"), NULL != lcp) {   /* becomes <---- */
+	while (lcp = strstr(lcstr, "<body>"), NULL != lcp) {
+		sp = returnstr + (lcp - lcstr) ; /* becomes <!   > */
+		sp++; *sp++ = '!'; *sp++ = ' '; *sp++ = ' '; *sp++ = ' ';
+		*lcp = '*';         /* so we wont find it again */
+	}
+	while (lcp = strstr(lcstr, "<body"), lcp != NULL) {   /* becomes <!--        --> */
 		sp = returnstr + (lcp - lcstr) ;
-		sp++; *sp++ = '-'; *sp++ = '-'; *sp++ = '-'; *sp++ = '-'; 
+		sp++; *sp++ = '!'; *sp++ = '-';  *sp++ = '-';  
+		while ( (*sp) && (*sp != '>') ) {
+		  sp++;
+		}
+		*--sp = '-'; *--sp = '-'; 
 		*lcp = '*';         /* so we wont find it again */
 	}
-	while (lcp = strstr(lcstr, "</body"), NULL != lcp) {
-		sp = returnstr + (lcp - lcstr) ; /* becomes </---- */
-		sp++; sp++; *sp++ = '-'; *sp++ = '-'; *sp++ = '-'; *sp++ = '-'; 
+	while (lcp = strstr(lcstr, "</body>"), NULL != lcp) {
+		sp = returnstr + (lcp - lcstr) ; /* becomes <!---- */
+		sp++; *sp++ = '!'; *sp++ = '-'; *sp++ = '-'; *sp++ = '-'; *sp++ = '-'; 
 		*lcp = '*';         /* so we wont find it again */
 	}
-	while (lcp = strstr(lcstr, "</html"), NULL != lcp) {
+	while (lcp = strstr(lcstr, "</html>"), NULL != lcp) {
 		sp = returnstr + (lcp - lcstr) ; /* becomes </---- */
-		sp++; sp++; *sp++ = '-'; *sp++ = '-'; *sp++ = '-'; *sp++ = '-'; 
+		sp++; *sp++ = '!'; *sp++ = '-'; *sp++ = '-'; *sp++ = '-'; *sp++ = '-'; 
 		*lcp = '*';         /* so we wont find it again */
 	}
 	while (lcp = strstr(lcstr, "<style"), NULL != lcp) {
